module PictureHandler
  module Uploader

    class Base

      include PictureHandler::Uploader::MiniMagick
      include PictureHandler::Uploader::ImgVersion
      include ('PictureHandler::Uploader::FileSystem::' + PictureHandler.configuration.file_sys.to_s.camelize).constantize

      attr_accessor :image, :model, :versions
      attr_reader :real_filename

      def initialize(model=nil)
        # Set once
        @image = nil
        @model = model
        @versions = {}
        register_versions

        # Changing runtime
        @current_version = nil
        @info_built = false
        @infos = {}
        @real_filename
      end

      def filename
        "unamed"
      end

      def store_dir
        "public/ph_pictures/"
      end

      def remote_path
        nil
      end

      def versions_info
        res = info
        res = res.select do |obj|
          obj == :position || (res[:versions].include? obj)
        end
        res
      end


      # ----------------------------- #
      # ------ Public Method -------- #
      # ----------------------------- #

      def info
        build_infos unless @info_built
        @infos
      end

      # When a Model.save is done
      def store

        # Get the filename once and will use the same for all futur save
        @real_filename = filename
        # Is the model new ? delete old record if true
        delete_all if @model.old_record

        process_original_image
        apply_versions

      end

      def delete_all
        delete_all_with_file_system
      end

      private

      # -------------------------- #
      # ------    INFOS     ------ #
      # -------------------------- #

      # At creation or called once before .info ?
      def build_infos
        model_filename = @model.model_filename
        infos = {}

        infos[:filename] = model_filename
        infos[:position] = model_filename.split('_').first.to_i
        infos[:store_dir] = store_dir
        infos[:original_path] = store_dir + model_filename
        infos[:versions] = []
        infos[:paths] = []
        infos[:paths] << { key: infos[:original_path] }
        # Get infos for each version
        @versions.each_key do |version|
          infos[:versions] << version
          version_path = store_dir + version.to_s + '_' + model_filename
          infos[:paths] << { key: version_path }
          infos[version] = version_path
        end
        @infos = infos
        @info_built = true
      end


      # -------------------------- #
      # ------   CREATE     ------ #
      # -------------------------- #

      def register_versions
        # Create different versions of your uploaded files:
        version :thumbnail do
          process resize_to_fill: [200, 200], quality: 80
        end
      end

      def process_original_image

        # Call the "SpecialUploader" remote_path method or the Uploader::Base one as default
        source_path = remote_path
        @image = self.class.open_source(source_path)

        path_to_original = store_dir + @real_filename + "." + @image.type.downcase
        write_file(@image, path_to_original)

      end

      def apply_versions

        # Get the different wanted versions
        @versions.each do |key, value|
          @current_version = key
          create_image_and_process( &(value[:block]) )
        end

      end

      # Create a new image for each versions or inherit it from the "from" if specified
      def create_image_and_process

        # Get if there is a from_version for this version
        from = @versions[@current_version][:from_version]

        # create a new image from original OR from another to be used for this version
        if from
          raise PictureHandler::Exceptions::VersionError.new(from) if @versions[from].blank? || @versions[from][:image].blank?
          # Put the version_image to our version also so both reference the same one
          image = @versions[from][:image]
          @versions[@current_version].merge!( {image: image} )

        else # Else create a new instance for this version
          # Open a new image from the local tmp location
          local_path = @image.path
          image = self.class.open_source(local_path)
          # Add the Image to this @versions[:version_name][:image]
          @versions[@current_version].merge!( {image: image} )
        end

        # Apply the options through the 'process' method
        yield if block_given?

        # Save the current version
        version_path = store_dir + @current_version.to_s + "_" + @real_filename + "." + image.type.downcase
        # Write this file new version
        write_file(image, version_path)
      end

    end

  end

end